@@ -62,7 +62,10 @@ else:
 import Widgets
 
 bindtextdomain(app_name, utils.get_locale(),locale_dir)
-       
+
+class Globals(super):
+    speed = _('Medium')
+
 class Window(SimpleGladeApp):
     def __init__(self, parent=None, path="gvr_gtk.glade", root="window_main", domain=app_name, **kwargs):
         path = os.path.join(glade_dir, path)
@@ -77,7 +80,7 @@ class Window(SimpleGladeApp):
                 self.parentGUI.set_icon_from_file(file)
             except gobject.GError:
                 self.logger.exception("Can't load window icon")
-            self.parentGUI.set_size_request(800,600)
+            #self.parentGUI.set_size_request(800,600)
             self.parentGUI.connect('delete_event',self.stop)
             
             # We setup key event callback here as WBCanvas can't recieve them ?? XXX
@@ -138,11 +141,16 @@ class Window(SimpleGladeApp):
         self.statusbar = Widgets.StatusBar(self.statusbar7)
         self._setup_canvas()
         # these calls will add the editors to the GUI
-        self.world_editor = WorldTextEditorWin(parent=self)
-        self.program_editor = CodeTextEditorWin(parent=self)
+        self.new_world_editor()
+        self.new_program_editor()
         self._set_sensitive_button('all',True)
         self.timerinterval = 150
-               
+    
+    def new_world_editor(self):
+        self.world_editor = WorldTextEditorWin(parent=self)
+    def new_program_editor(self):
+        self.program_editor = CodeTextEditorWin(parent=self)
+    
     def _setup_canvas(self):
         # setup the canvas
         self._canvas = Widgets.Canvas()
@@ -178,11 +186,12 @@ class Window(SimpleGladeApp):
     def start(self,args):
         """This will start the GUI."""
         self.logger.debug("start called with args: %s,%s" % args)
+        # there are only args when gvr is started by gvrng.py (non-XO systems)
         if args[0] and args[1]:
             wfile, pfile = args[0], args[1]
-            self.on_new_world1_activate(file=wfile)
-            self.on_new_program1_activate(file=pfile)
-        # Sugar runs the gtk loop
+            self.world_editor.on_new1_activate(file=wfile)
+            self.program_editor.on_new1_activate(file=pfile)
+        # Sugar runs the gtk loop on XO
         if PLATFORM != 'XO':
             self.parentGUI.show()
             self.run()
@@ -269,8 +278,9 @@ class Window(SimpleGladeApp):
     def on_open_worldbuilder1_activate(self, widget, *args):
         self.logger.debug("worldbuilder_activate")
         self.WB_ACTIVATED = True
-        self.windowtitle = self.parentGUI.get_title()
-        self.parentGUI.set_title(_("GvR - Worldbuilder"))
+        if PLATFORM != 'XO':
+            self.windowtitle = self.parentGUI.get_title()
+            self.parentGUI.set_title(_("GvR - Worldbuilder"))
         
         # first we disable all the buttons.
         self._set_sensitive_button('all',False)
@@ -377,6 +387,7 @@ class Window(SimpleGladeApp):
         if wcode:
             self.controller.on_button_reload(wcode)
         self.program_editor.reset_highlight()
+        self._canvas._reset_offset()
         return True
 
     def on_button_step(self, widget, *args):
@@ -392,7 +403,8 @@ class Window(SimpleGladeApp):
             # we act now as a 'quit button for the WB
             # canvas is now a WBCanvas object
             # Reset everything for the 'normal' canvas
-            self.parentGUI.set_title(self.windowtitle)
+            if PLATFORM != 'XO':
+                self.parentGUI.set_title(self.windowtitle)
             self._set_sensitive_button('all',False)
             self.scrolledwindow8.remove(self.viewport)
             # put back the gvr world canvas
@@ -538,7 +550,7 @@ class TextEditorWin(SimpleGladeApp):
 
     def new(self):
         """This implements a gtksourceview widget from Editors.py"""
-        #print "A new TextEditorWin has been created"
+        #Called by SimpleGladeApp
         # We use different editors on win32 as gtksourceview isn't available
         # Editors can be Editors.py or Win_Editors.py, see import statement above
         self.editor = Editors.Editor(self.scrolledwindow1)
@@ -633,19 +645,6 @@ class TextEditorWin(SimpleGladeApp):
                 self.on_save_as1_activate(txt=edittxt)
             else:
                 return True
-##        self.EditorWin.destroy() 
-##        self.loaded_txt = []
-##        self.editor = None
-##        if str(self) == 'wld':
-##            self.parent._set_sensitive_button('reload',False)
-##        else:
-##            for b in ('execute','step','abort'):
-##                self.parent._set_sensitive_button(b,False)
-##        try:
-##            self.parent.canvas.draw_splash()
-##        except:
-##            pass
-##        self._notify_observers()
    
     def on_cut1_activate(self, widget, *args):
         self.editor.srcview.emit('cut-clipboard')
@@ -689,79 +688,91 @@ class CodeTextEditorWin(TextEditorWin):
         TextEditorWin.__init__(self,path,root,domain,parent,**kwargs)
         self.parent = parent
         self.EditorWin.remove(self.vbox4)
+        # make sure we don't add > 1 children
+        for child in self.parent.alignment19.get_children():
+            self.parent.alignment19.remove(child)
         self.parent.alignment19.add(self.vbox4)
     def __str__(self):
         return 'gvr'
     
     def on_new1_activate(self, widget=None, file=''):
-        if file:
-            txt = utils.load_file(file)
-            if txt:
-                self.set_text(file,txt)
-        #self.program_editor.EditorWin.show()
-        for b in ('execute','step','abort'):
-            self.parent._set_sensitive_button(b,True)
-        self.register_observer(self.parent._programeditor_observer_callback)
+        self.parent.new_program_editor()
+##        if file:
+##            txt = utils.load_file(file)
+##            if txt:
+##                self.set_text(file,txt)
+##        #self.program_editor.EditorWin.show()
+##        for b in ('execute','step','abort'):
+##            self.parent._set_sensitive_button(b,True)
+##        self.register_observer(self.parent._programeditor_observer_callback)
     
     def on_open1_activate(self, widget=None,file=''):
-        if file:
-            self.on_new_program1_activate(widget,file)
-            return
-        path = ''
-        dlg = FileDialog(action='open',title=_("Open GvR program"),ext='gvr')
-        response = dlg.run()
-        if response == gtk.RESPONSE_OK:
-            path = dlg.get_filename()
-            if os.path.splitext(path)[1] != '.gvr':
-                self.show_error(_("Selected path is not a program file"))
+        if not file:
+            dlg = FileDialog(action='open',title=_("Open GvR program"),ext='gvr')
+            response = dlg.run()
+            if response == gtk.RESPONSE_OK:
+                file = dlg.get_filename()
+                if os.path.splitext(file)[1] != '.gvr':
+                    self.show_error(_("Selected path is not a program file"))
+                    dlg.destroy()
+                    return
+            elif response == gtk.RESPONSE_CANCEL:
+                self.logger.debug('Closed, no files selected')
                 dlg.destroy()
                 return
-        elif response == gtk.RESPONSE_CANCEL:
-            self.logger.debug('Closed, no files selected')
             dlg.destroy()
-            return
-        dlg.destroy()
-        self.on_new1_activate(self,path)
-    
-    
-    
+        txt = utils.load_file(file)
+        if txt:
+            self.set_text(file,txt)
+        for b in ('execute','step','abort'):
+            self.parent._set_sensitive_button(b,True)
+        return
+       
 class WorldTextEditorWin(TextEditorWin):
     def __init__(self, path="gvr_gtk.glade", root="EditorWin", domain=app_name,parent=None, **kwargs):
         TextEditorWin.__init__(self,path,root,domain,parent,**kwargs)
         self.parent = parent
         self.EditorWin.remove(self.vbox4)
+        # make sure we don't add > 1 children
+        for child in self.parent.alignment18.get_children():
+            self.parent.alignment18.remove(child)
         self.parent.alignment18.add(self.vbox4)
     def __str__(self):
         return 'wld'
     
     def on_new1_activate(self,widget=None,file=''):
-        if file:
-            txt = utils.load_file(file)
-            if txt:
-                self.set_text(file,txt)
-                self.parent.on_button_reload()
-        self.parent._set_sensitive_button('reload',True)
-        self.register_observer(self.parent._worldeditor_observer_callback)
+        self.parent.new_world_editor()
+##        self.on_quit2_activate()# this takes care of saving content yes/no
+##        #self.new()
+##        if file:
+##            txt = utils.load_file(file)
+##            if txt:
+##                self.set_text(file,txt)
+##                self.parent.on_button_reload()
+##        self.parent._set_sensitive_button('reload',True)
+##        self.register_observer(self.parent._worldeditor_observer_callback)
     
     def on_open1_activate(self,widget=None,file=''):
-        if file:
-            self.on_new_world1_activate(widget,file)
-            return
-        path = ''
-        dlg = FileDialog(action='open',title=_("Open GvR world"),ext='wld')
-        response = dlg.run()
-        if response == gtk.RESPONSE_OK:
-            path = dlg.get_filename()
-            if os.path.splitext(path)[1] != '.wld':
-                self.show_error(_("Selected path is not a world file"))
+        self.on_quit2_activate()# this takes care of saving content yes/no
+        if not file:
+            dlg = FileDialog(action='open',title=_("Open GvR world"),ext='wld')
+            response = dlg.run()
+            if response == gtk.RESPONSE_OK:
+                file = dlg.get_filename()
+                if os.path.splitext(file)[1] != '.wld':
+                    self.show_error(_("Selected path is not a world file"))
+                    dlg.destroy()
+                    return
+            elif response == gtk.RESPONSE_CANCEL:
+                self.logger.debug('Closed, no files selected')
                 dlg.destroy()
                 return
-        elif response == gtk.RESPONSE_CANCEL:
-            self.logger.debug('Closed, no files selected')
             dlg.destroy()
-            return
-        dlg.destroy()
-        self.on_new1_activate(self,path)
+        txt = utils.load_file(file)
+        if txt:
+            self.set_text(file,txt)
+            self.parent.on_button_reload()
+        return
         
 class SetLanguageDialog(SimpleGladeApp):
     def __init__(self, path="gvr_gtk.glade", root="SetLanguageDialog", domain=app_name, **kwargs):
@@ -794,15 +805,17 @@ class SetSpeedDialog(SimpleGladeApp):
         SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
 
     def new(self):
-        self.comboboxentry_speed.set_active(2)
+        choice = {_('Instant'):0,_('Fast'):1,_('Medium'):2,_('Slow'):3}[Globals.speed]
+        self.comboboxentry_speed.set_active(choice)
     
     def get_choice(self):
         try:
-            choice = {'Instant':5,'Fast':50,'Medium':150,'Slow':500}\
-                    [Widgets.get_active_text(self.comboboxentry_speed)]
+            txt = Widgets.get_active_text(self.comboboxentry_speed)
+            choice = {'Instant':5,'Fast':50,'Medium':150,'Slow':500}[txt]
         except Exception,info:
             print info
             choice = 150
+        Globals.speed = _(txt)
         return choice
     
     def on_SetSpeedDialog_delete_event(self, widget, *args):
