@@ -12,5 +12,5 @@ cp $TEMPDIR/messages.po GvRng_$VERS.pot
 find . -name "*.po" | while read pofile
     do
 	echo $pofile
-        msgmerge --update --backup=off -i -F "$pofile" $TEMPDIR/messages.po
+        msgmerge --update --backup=off -i -s $pofile $TEMPDIR/messages.po
     done
